/*
  - name: english
  - author: martin fierz, peter chiocchetti
  - purpose: platform independent checkers move validator
  - version: 1
  - date: 4th december 2004
  - description: english.c checks moves for validity,
    this one is for english draughts, or american checkers
    code is split into interface and move generation.

  - board representation: the standard checkers notation is

    	(white)
      32  31  30  29
    28  27  26  25
      24  23  22  21
    20  19  18  17
      16  15  14  13
    12  11  10   9
       8   7   6   5
     4   3   2   1
    	(black)

    the internal representation of the board is different, it is
    an array of int with length 46, the checkers board is numbered
    like this:

    	(white)
      37  38  39  40
    32  33  34  35
      28  29  30  31
    23  24  25  26
      19  20  21  22
    14  15  16  17
      10  11  12  13
     5   6   7   8
    	(black)
*/


/* includes */
#include <stdio.h>
#include <string.h>

/* definitions */
#define OCCUPIED 0
#define WHITE 1
#define BLACK 2
#define MAN 4
#define KING 8
#define FREE 16
#define MAXMOVES 28
#define SQUARE 12		// max rows, cols, canadian board


/* structure definitions  */
struct move2
{
  short n;
  int m[8];
};

/* coordinate structure for board coordinates */
struct coor
{
  int x;
  int y;
};

/* all the information you need about a move */
struct CBmove
{
  int ismove;			// kind of superfluous: is 0 if the move is not valid
  int newpiece;			// what type of piece appears on to
  int oldpiece;			// what disappears on from
  struct coor from, to;		// coordinates of the piece - in 8x8 notation!
  struct coor path[12];		// intermediate path coordinates of the moving piece
  struct coor del[12];		// squares whose pieces are deleted after the move
  int delpiece[12];		// what is on these squares
} GCBmove;


/* function prototypes  */
/* interface  */
int islegal_english (int b[SQUARE][SQUARE], int color, int pdmove[8],
		     struct CBmove *move);
void setbestmove (struct move2 move);
struct coor numbertocoor (int n);
int movetonotation (struct move2 move, int piece);

/* move generation */
int generatemovelist (int b[46], struct move2 movelist[MAXMOVES], int color);
int generatecapturelist (int b[46], struct move2 movelist[MAXMOVES],
			 int color);
void blackmancapture (int b[46], int *n, struct move2 movelist[MAXMOVES],
		      int square);
void blackkingcapture (int b[46], int *n, struct move2 movelist[MAXMOVES],
		       int square);
void whitemancapture (int b[46], int *n, struct move2 movelist[MAXMOVES],
		      int square);
void whitekingcapture (int b[46], int *n, struct move2 movelist[MAXMOVES],
		       int square);



/* INTERFACE */

int
islegal_english (int b[SQUARE][SQUARE], int color, int pdnmove[8],
		 struct CBmove *move)
{
  /* to check if a move the user wants to make is legal or not,
     we generate a movelist and compare the moves in the movelist
     to the move the user wants to make with from&to */

  int n, i, j, found = 0;
  struct move2 movelist[MAXMOVES];
  int board[46];
  int capture = 0;
  int from = 0, to = 0;

  /* initialize board */
  for (i = 0; i < 46; i++)
    board[i] = OCCUPIED;
  for (i = 5; i <= 40; i++)
    board[i] = FREE;
  board[5] = b[0][0];
  board[6] = b[2][0];
  board[7] = b[4][0];
  board[8] = b[6][0];
  board[10] = b[1][1];
  board[11] = b[3][1];
  board[12] = b[5][1];
  board[13] = b[7][1];
  board[14] = b[0][2];
  board[15] = b[2][2];
  board[16] = b[4][2];
  board[17] = b[6][2];
  board[19] = b[1][3];
  board[20] = b[3][3];
  board[21] = b[5][3];
  board[22] = b[7][3];
  board[23] = b[0][4];
  board[24] = b[2][4];
  board[25] = b[4][4];
  board[26] = b[6][4];
  board[28] = b[1][5];
  board[29] = b[3][5];
  board[30] = b[5][5];
  board[31] = b[7][5];
  board[32] = b[0][6];
  board[33] = b[2][6];
  board[34] = b[4][6];
  board[35] = b[6][6];
  board[37] = b[1][7];
  board[38] = b[3][7];
  board[39] = b[5][7];
  board[40] = b[7][7];
  for (i = 5; i <= 40; i++)
    if (board[i] == 0)
      board[i] = FREE;
  for (i = 9; i <= 36; i += 9)
    board[i] = OCCUPIED;
  /* board initialized */


  n = generatecapturelist (board, movelist, color);
  capture = n;

  if (!n)
    n = generatemovelist (board, movelist, color);
  if (!n)
    return 0;

  /* now we have a movelist - check if from and to are the same */
  for (i = 0; i < n && !found; i++)
    {
      from = movetonotation (movelist[i], movelist[i].m[0]);
      to = movetonotation (movelist[i], movelist[i].m[1]);

      // from and to match?
      if (from == pdnmove[1] && to == pdnmove[pdnmove[0]])
	found = 1;
      // match and only from-to given?
      if (found && pdnmove[0] == 2)
	break;
      // no match and only from-to given?
      if (!found || pdnmove[0] == 2)
	continue;

      /* test intermediate squares
         bound by both movelist and pdnmove count */
      found = 0;
      for (j = 2; j < movelist[i].n && j <= pdnmove[0]; j++)
	{
	  int mid, end = 0;
	  // for all deleted pieces
	  mid = movetonotation (movelist[i], movelist[i].m[j]);
	  // find the landing square by board number magic
	  switch (mid - from)
	    {
	    case -5:
	      end = mid - 4;
	      break;
	    case -4:
	      end = from % 8 > 4 ? mid - 3 : mid - 5;
	      break;
	    case -3:
	      end = mid - 4;
	      break;
	    case 3:
	      end = mid + 4;
	      break;
	    case 4:
	      end = from % 8 > 4 ? mid + 5 : mid + 3;
	      break;
	    case 5:
	      end = mid + 4;
	      break;
	    }
	  // a miss
	  if (end != pdnmove[j])
	    break;
	  from = end;
	}
      // all squares matched?
      if (j == movelist[i].n)
	{
	  found = 1;
	  break;
	}
    }

  if (found)
    /* sets GCBmove to movelist[i] */
    setbestmove (movelist[i]);

  *move = GCBmove;
  return found;
}


void
setbestmove (struct move2 move)
{
  int i;
  int jumps;
  int from, to;
  struct coor c1, c2;

  jumps = move.n - 2;

  from = move.m[0] % 256;
  to = move.m[1] % 256;

  GCBmove.from = numbertocoor (from);
  GCBmove.to = numbertocoor (to);
  GCBmove.ismove = jumps;
  GCBmove.newpiece = ((move.m[1] >> 16) % 256);
  GCBmove.oldpiece = ((move.m[0] >> 8) % 256);
  for (i = 2; i < move.n; i++)
    {
      GCBmove.delpiece[i - 2] = ((move.m[i] >> 8) % 256);
      GCBmove.del[i - 2] = numbertocoor (move.m[i] % 256);
    }
  if (jumps > 1)
    /* more than one jump - need to calculate intermediate squares */
    {
      /* set square where we start to c1 */
      c1 = numbertocoor (from);
      for (i = 2; i < move.n; i++)
	{
	  c2 = numbertocoor (move.m[i] % 256);
	  /* c2 is the piece we jump
	     => we land on the next square?! */
	  if (c2.x > c1.x)
	    c2.x++;
	  else
	    c2.x--;
	  if (c2.y > c1.y)
	    c2.y++;
	  else
	    c2.y--;
	  /* now c2 holds the square after the jumped piece
	     this is our path square */
	  GCBmove.path[i - 1] = c2;
	  c1 = c2;
	}
    }
  else
    {
      GCBmove.path[1] = numbertocoor (to);
    }
}


struct coor
numbertocoor (int n)
{
  /* turns square number n into a coordinate for checkerboard */
  /*
     (white)
     37  38  39  40
     32  33  34  35
     28  29  30  31
     23  24  25  26
     19  20  21  22
     14  15  16  17
     10  11  12  13
     5   6   7   8
     (black)
   */
  struct coor c;
  switch (n)
    {
    case 5:
      c.x = 0;
      c.y = 0;
      break;
    case 6:
      c.x = 2;
      c.y = 0;
      break;
    case 7:
      c.x = 4;
      c.y = 0;
      break;
    case 8:
      c.x = 6;
      c.y = 0;
      break;
    case 10:
      c.x = 1;
      c.y = 1;
      break;
    case 11:
      c.x = 3;
      c.y = 1;
      break;
    case 12:
      c.x = 5;
      c.y = 1;
      break;
    case 13:
      c.x = 7;
      c.y = 1;
      break;
      /*
         (white)
         37  38  39  40
         32  33  34  35
         28  29  30  31
         23  24  25  26
         19  20  21  22
         14  15  16  17
         10  11  12  13
         5   6   7   8
         (black)
       */
    case 14:
      c.x = 0;
      c.y = 2;
      break;
    case 15:
      c.x = 2;
      c.y = 2;
      break;
    case 16:
      c.x = 4;
      c.y = 2;
      break;
    case 17:
      c.x = 6;
      c.y = 2;
      break;
    case 19:
      c.x = 1;
      c.y = 3;
      break;
    case 20:
      c.x = 3;
      c.y = 3;
      break;
    case 21:
      c.x = 5;
      c.y = 3;
      break;
    case 22:
      c.x = 7;
      c.y = 3;
      break;
      /*
         (white)
         37  38  39  40
         32  33  34  35
         28  29  30  31
         23  24  25  26
         19  20  21  22
         14  15  16  17
         10  11  12  13
         5   6   7   8
         (black)
       */
    case 23:
      c.x = 0;
      c.y = 4;
      break;
    case 24:
      c.x = 2;
      c.y = 4;
      break;
    case 25:
      c.x = 4;
      c.y = 4;
      break;
    case 26:
      c.x = 6;
      c.y = 4;
      break;
    case 28:
      c.x = 1;
      c.y = 5;
      break;
    case 29:
      c.x = 3;
      c.y = 5;
      break;
    case 30:
      c.x = 5;
      c.y = 5;
      break;
    case 31:
      c.x = 7;
      c.y = 5;
      break;
      /*
         (white)
         37  38  39  40
         32  33  34  35
         28  29  30  31
         23  24  25  26
         19  20  21  22
         14  15  16  17
         10  11  12  13
         5   6   7   8
         (black)
       */
    case 32:
      c.x = 0;
      c.y = 6;
      break;
    case 33:
      c.x = 2;
      c.y = 6;
      break;
    case 34:
      c.x = 4;
      c.y = 6;
      break;
    case 35:
      c.x = 6;
      c.y = 6;
      break;
    case 37:
      c.x = 1;
      c.y = 7;
      break;
    case 38:
      c.x = 3;
      c.y = 7;
      break;
    case 39:
      c.x = 5;
      c.y = 7;
      break;
    case 40:
      c.x = 7;
      c.y = 7;
      break;
    }
  /*
     (white)
     37  38  39  40
     32  33  34  35
     28  29  30  31
     23  24  25  26
     19  20  21  22
     14  15  16  17
     10  11  12  13
     5   6   7   8
     (black)
   */
  return c;
}


int
movetonotation (struct move2 move, int piece)
{
  int j;

  piece = piece % 256;
  piece = piece - (piece / 9);
  piece -= 5;
  j = piece % 4;
  piece -= j;
  j = 3 - j;
  piece += j;
  piece++;
  return piece;
}


/* MOVE GENERATION */

int
generatemovelist (int b[46], struct move2 movelist[MAXMOVES], int color)
/* - purpose:generates all moves. no captures. returns number of moves
   - version: 1.0
   - date: 25th october 97 */
{
  int n = 0, m;
  int i;

  if (color == BLACK)
    {
      for (i = 5; i <= 40; i++)
	{
	  if ((b[i] & BLACK) != 0)
	    {
	      if ((b[i] & MAN) != 0)
		{
		  if ((b[i + 4] & FREE) != 0)
		    {
		      movelist[n].n = 2;
		      if (i >= 32)
			m = (BLACK | KING);
		      else
			m = (BLACK | MAN);
		      m = m << 8;
		      m += FREE;
		      m = m << 8;
		      m += i + 4;
		      movelist[n].m[1] = m;
		      m = FREE;
		      m = m << 8;
		      m += (BLACK | MAN);
		      m = m << 8;
		      m += i;
		      movelist[n].m[0] = m;
		      n++;
		    }
		  if ((b[i + 5] & FREE) != 0)
		    {
		      movelist[n].n = 2;
		      if (i >= 32)
			m = (BLACK | KING);
		      else
			m = (BLACK | MAN);
		      m = m << 8;
		      m += FREE;
		      m = m << 8;
		      m += i + 5;
		      movelist[n].m[1] = m;
		      m = FREE;
		      m = m << 8;
		      m += (BLACK | MAN);
		      m = m << 8;
		      m += i;
		      movelist[n].m[0] = m;
		      n++;
		    }
		}
	      if ((b[i] & KING) != 0)
		{
		  if ((b[i + 4] & FREE) != 0)
		    {
		      movelist[n].n = 2;
		      m = (BLACK | KING);
		      m = m << 8;
		      m += FREE;
		      m = m << 8;
		      m += i + 4;
		      movelist[n].m[1] = m;
		      m = FREE;
		      m = m << 8;
		      m += (BLACK | KING);
		      m = m << 8;
		      m += i;
		      movelist[n].m[0] = m;
		      n++;
		    }
		  if ((b[i + 5] & FREE) != 0)
		    {
		      movelist[n].n = 2;
		      m = (BLACK | KING);
		      m = m << 8;
		      m += FREE;
		      m = m << 8;
		      m += i + 5;
		      movelist[n].m[1] = m;
		      m = FREE;
		      m = m << 8;
		      m += (BLACK | KING);
		      m = m << 8;
		      m += i;
		      movelist[n].m[0] = m;
		      n++;
		    }
		  if ((b[i - 4] & FREE) != 0)
		    {
		      movelist[n].n = 2;
		      m = (BLACK | KING);
		      m = m << 8;
		      m += FREE;
		      m = m << 8;
		      m += i - 4;
		      movelist[n].m[1] = m;
		      m = FREE;
		      m = m << 8;
		      m += (BLACK | KING);
		      m = m << 8;
		      m += i;
		      movelist[n].m[0] = m;
		      n++;
		    }
		  if ((b[i - 5] & FREE) != 0)
		    {
		      movelist[n].n = 2;
		      m = (BLACK | KING);
		      m = m << 8;
		      m += FREE;
		      m = m << 8;
		      m += i - 5;
		      movelist[n].m[1] = m;
		      m = FREE;
		      m = m << 8;
		      m += (BLACK | KING);
		      m = m << 8;
		      m += i;
		      movelist[n].m[0] = m;
		      n++;
		    }
		}
	    }
	}
    }
  else				/* color = WHITE */
    {
      for (i = 5; i <= 40; i++)
	{
	  if ((b[i] & WHITE) != 0)
	    {
	      if ((b[i] & MAN) != 0)
		{
		  if ((b[i - 4] & FREE) != 0)
		    {
		      movelist[n].n = 2;
		      if (i <= 13)
			m = (WHITE | KING);
		      else
			m = (WHITE | MAN);
		      m = m << 8;
		      m += FREE;
		      m = m << 8;
		      m += i - 4;
		      movelist[n].m[1] = m;
		      m = FREE;
		      m = m << 8;
		      m += (WHITE | MAN);
		      m = m << 8;
		      m += i;
		      movelist[n].m[0] = m;
		      n++;
		    }
		  if ((b[i - 5] & FREE) != 0)
		    {
		      movelist[n].n = 2;
		      if (i <= 13)
			m = (WHITE | KING);
		      else
			m = (WHITE | MAN);
		      m = m << 8;
		      m += FREE;
		      m = m << 8;
		      m += i - 5;
		      movelist[n].m[1] = m;
		      m = FREE;
		      m = m << 8;
		      m += (WHITE | MAN);
		      m = m << 8;
		      m += i;
		      movelist[n].m[0] = m;
		      n++;
		    }
		}
	      if ((b[i] & KING) != 0)	/* or else */
		{
		  if ((b[i + 4] & FREE) != 0)
		    {
		      movelist[n].n = 2;
		      m = (WHITE | KING);
		      m = m << 8;
		      m += FREE;
		      m = m << 8;
		      m += i + 4;
		      movelist[n].m[1] = m;
		      m = FREE;
		      m = m << 8;
		      m += (WHITE | KING);
		      m = m << 8;
		      m += i;
		      movelist[n].m[0] = m;
		      n++;
		    }
		  if ((b[i + 5] & FREE) != 0)
		    {
		      movelist[n].n = 2;
		      m = (WHITE | KING);
		      m = m << 8;
		      m += FREE;
		      m = m << 8;
		      m += i + 5;
		      movelist[n].m[1] = m;
		      m = FREE;
		      m = m << 8;
		      m += (WHITE | KING);
		      m = m << 8;
		      m += i;
		      movelist[n].m[0] = m;
		      n++;
		    }
		  if ((b[i - 4] & FREE) != 0)
		    {
		      movelist[n].n = 2;
		      m = (WHITE | KING);
		      m = m << 8;
		      m += FREE;
		      m = m << 8;
		      m += i - 4;
		      movelist[n].m[1] = m;
		      m = FREE;
		      m = m << 8;
		      m += (WHITE | KING);
		      m = m << 8;
		      m += i;
		      movelist[n].m[0] = m;
		      n++;
		    }
		  if ((b[i - 5] & FREE) != 0)
		    {
		      movelist[n].n = 2;
		      m = (WHITE | KING);
		      m = m << 8;
		      m += FREE;
		      m = m << 8;
		      m += i - 5;
		      movelist[n].m[1] = m;
		      m = FREE;
		      m = m << 8;
		      m += (WHITE | KING);
		      m = m << 8;
		      m += i;
		      movelist[n].m[0] = m;
		      n++;
		    }
		}
	    }
	}
    }
  return (n);
}

int
generatecapturelist (int b[46], struct move2 movelist[MAXMOVES], int color)
/* - purpose: generate all possible captures
   - version: 1.0
   - date: 25th october 97 */
{
  int n = 0;
  int m;
  int i;
  int tmp;

  if (color == BLACK)
    {
      for (i = 5; i <= 40; i++)
	{
	  if ((b[i] & BLACK) != 0)
	    {
	      if ((b[i] & MAN) != 0)
		{
		  if ((b[i + 4] & WHITE) != 0)
		    {
		      if ((b[i + 8] & FREE) != 0)
			{
			  movelist[n].n = 3;
			  if (i >= 28)
			    m = (BLACK | KING);
			  else
			    m = (BLACK | MAN);
			  m = m << 8;
			  m += FREE;
			  m = m << 8;
			  m += i + 8;
			  movelist[n].m[1] = m;
			  m = FREE;
			  m = m << 8;
			  m += (BLACK | MAN);
			  m = m << 8;
			  m += i;
			  movelist[n].m[0] = m;
			  m = FREE;
			  m = m << 8;
			  m += b[i + 4];
			  m = m << 8;
			  m += i + 4;
			  movelist[n].m[2] = m;
			  blackmancapture (b, &n, movelist, i + 8);
			}
		    }
		  if ((b[i + 5] & WHITE) != 0)
		    {
		      if ((b[i + 10] & FREE) != 0)
			{
			  movelist[n].n = 3;
			  if (i >= 28)
			    m = (BLACK | KING);
			  else
			    m = (BLACK | MAN);
			  m = m << 8;
			  m += FREE;
			  m = m << 8;
			  m += i + 10;
			  movelist[n].m[1] = m;
			  m = FREE;
			  m = m << 8;
			  m += (BLACK | MAN);
			  m = m << 8;
			  m += i;
			  movelist[n].m[0] = m;
			  m = FREE;
			  m = m << 8;
			  m += b[i + 5];
			  m = m << 8;
			  m += i + 5;
			  movelist[n].m[2] = m;
			  blackmancapture (b, &n, movelist, i + 10);
			}
		    }
		}
	      else		/* b[i] is a KING */
		{
		  if ((b[i + 4] & WHITE) != 0)
		    {
		      if ((b[i + 8] & FREE) != 0)
			{
			  movelist[n].n = 3;
			  m = (BLACK | KING);
			  m = m << 8;
			  m += FREE;
			  m = m << 8;
			  m += i + 8;
			  movelist[n].m[1] = m;
			  m = FREE;
			  m = m << 8;
			  m += (BLACK | KING);
			  m = m << 8;
			  m += i;
			  movelist[n].m[0] = m;
			  m = FREE;
			  m = m << 8;
			  m += b[i + 4];
			  m = m << 8;
			  m += i + 4;
			  movelist[n].m[2] = m;
			  tmp = b[i + 4];
			  b[i + 4] = FREE;
			  blackkingcapture (b, &n, movelist, i + 8);
			  b[i + 4] = tmp;
			}
		    }
		  if ((b[i + 5] & WHITE) != 0)
		    {
		      if ((b[i + 10] & FREE) != 0)
			{
			  movelist[n].n = 3;
			  m = (BLACK | KING);
			  m = m << 8;
			  m += FREE;
			  m = m << 8;
			  m += i + 10;
			  movelist[n].m[1] = m;
			  m = FREE;
			  m = m << 8;
			  m += (BLACK | KING);
			  m = m << 8;
			  m += i;
			  movelist[n].m[0] = m;
			  m = FREE;
			  m = m << 8;
			  m += b[i + 5];
			  m = m << 8;
			  m += i + 5;
			  movelist[n].m[2] = m;
			  tmp = b[i + 5];
			  b[i + 5] = FREE;
			  blackkingcapture (b, &n, movelist, i + 10);
			  b[i + 5] = tmp;
			}
		    }
		  if ((b[i - 4] & WHITE) != 0)
		    {
		      if ((b[i - 8] & FREE) != 0)
			{
			  movelist[n].n = 3;
			  m = (BLACK | KING);
			  m = m << 8;
			  m += FREE;
			  m = m << 8;
			  m += i - 8;
			  movelist[n].m[1] = m;
			  m = FREE;
			  m = m << 8;
			  m += (BLACK | KING);
			  m = m << 8;
			  m += i;
			  movelist[n].m[0] = m;
			  m = FREE;
			  m = m << 8;
			  m += b[i - 4];
			  m = m << 8;
			  m += i - 4;
			  movelist[n].m[2] = m;
			  tmp = b[i - 4];
			  b[i - 4] = FREE;
			  blackkingcapture (b, &n, movelist, i - 8);
			  b[i - 4] = tmp;
			}
		    }
		  if ((b[i - 5] & WHITE) != 0)
		    {
		      if ((b[i - 10] & FREE) != 0)
			{
			  movelist[n].n = 3;
			  m = (BLACK | KING);
			  m = m << 8;
			  m += FREE;
			  m = m << 8;
			  m += i - 10;
			  movelist[n].m[1] = m;
			  m = FREE;
			  m = m << 8;
			  m += (BLACK | KING);
			  m = m << 8;
			  m += i;
			  movelist[n].m[0] = m;
			  m = FREE;
			  m = m << 8;
			  m += b[i - 5];
			  m = m << 8;
			  m += i - 5;
			  movelist[n].m[2] = m;
			  tmp = b[i - 5];
			  b[i - 5] = FREE;
			  blackkingcapture (b, &n, movelist, i - 10);
			  b[i - 5] = tmp;
			}
		    }
		}
	    }
	}
    }
  else				/* color is WHITE */
    {
      for (i = 5; i <= 40; i++)
	{
	  if ((b[i] & WHITE) != 0)
	    {
	      if ((b[i] & MAN) != 0)
		{
		  if ((b[i - 4] & BLACK) != 0)
		    {
		      if ((b[i - 8] & FREE) != 0)
			{
			  movelist[n].n = 3;
			  if (i <= 17)
			    m = (WHITE | KING);
			  else
			    m = (WHITE | MAN);
			  m = m << 8;
			  m += FREE;
			  m = m << 8;
			  m += i - 8;
			  movelist[n].m[1] = m;
			  m = FREE;
			  m = m << 8;
			  m += (WHITE | MAN);
			  m = m << 8;
			  m += i;
			  movelist[n].m[0] = m;
			  m = FREE;
			  m = m << 8;
			  m += b[i - 4];
			  m = m << 8;
			  m += i - 4;
			  movelist[n].m[2] = m;
			  whitemancapture (b, &n, movelist, i - 8);
			}
		    }
		  if ((b[i - 5] & BLACK) != 0)
		    {
		      if ((b[i - 10] & FREE) != 0)
			{
			  movelist[n].n = 3;
			  if (i <= 17)
			    m = (WHITE | KING);
			  else
			    m = (WHITE | MAN);
			  m = m << 8;
			  m += FREE;
			  m = m << 8;
			  m += i - 10;
			  movelist[n].m[1] = m;
			  m = FREE;
			  m = m << 8;
			  m += (WHITE | MAN);
			  m = m << 8;
			  m += i;
			  movelist[n].m[0] = m;
			  m = FREE;
			  m = m << 8;
			  m += b[i - 5];
			  m = m << 8;
			  m += i - 5;
			  movelist[n].m[2] = m;
			  whitemancapture (b, &n, movelist, i - 10);
			}
		    }
		}
	      else		/* b[i] is a KING */
		{
		  if ((b[i + 4] & BLACK) != 0)
		    {
		      if ((b[i + 8] & FREE) != 0)
			{
			  movelist[n].n = 3;
			  m = (WHITE | KING);
			  m = m << 8;
			  m += FREE;
			  m = m << 8;
			  m += i + 8;
			  movelist[n].m[1] = m;
			  m = FREE;
			  m = m << 8;
			  m += (WHITE | KING);
			  m = m << 8;
			  m += i;
			  movelist[n].m[0] = m;
			  m = FREE;
			  m = m << 8;
			  m += b[i + 4];
			  m = m << 8;
			  m += i + 4;
			  movelist[n].m[2] = m;
			  tmp = b[i + 4];
			  b[i + 4] = FREE;
			  whitekingcapture (b, &n, movelist, i + 8);
			  b[i + 4] = tmp;
			}
		    }
		  if ((b[i + 5] & BLACK) != 0)
		    {
		      if ((b[i + 10] & FREE) != 0)
			{
			  movelist[n].n = 3;
			  m = (WHITE | KING);
			  m = m << 8;
			  m += FREE;
			  m = m << 8;
			  m += i + 10;
			  movelist[n].m[1] = m;
			  m = FREE;
			  m = m << 8;
			  m += (WHITE | KING);
			  m = m << 8;
			  m += i;
			  movelist[n].m[0] = m;
			  m = FREE;
			  m = m << 8;
			  m += b[i + 5];
			  m = m << 8;
			  m += i + 5;
			  movelist[n].m[2] = m;
			  tmp = b[i + 5];
			  b[i + 5] = FREE;
			  whitekingcapture (b, &n, movelist, i + 10);
			  b[i + 5] = tmp;
			}
		    }
		  if ((b[i - 4] & BLACK) != 0)
		    {
		      if ((b[i - 8] & FREE) != 0)
			{
			  movelist[n].n = 3;
			  m = (WHITE | KING);
			  m = m << 8;
			  m += FREE;
			  m = m << 8;
			  m += i - 8;
			  movelist[n].m[1] = m;
			  m = FREE;
			  m = m << 8;
			  m += (WHITE | KING);
			  m = m << 8;
			  m += i;
			  movelist[n].m[0] = m;
			  m = FREE;
			  m = m << 8;
			  m += b[i - 4];
			  m = m << 8;
			  m += i - 4;
			  movelist[n].m[2] = m;
			  tmp = b[i - 4];
			  b[i - 4] = FREE;
			  whitekingcapture (b, &n, movelist, i - 8);
			  b[i - 4] = tmp;
			}
		    }
		  if ((b[i - 5] & BLACK) != 0)
		    {
		      if ((b[i - 10] & FREE) != 0)
			{
			  movelist[n].n = 3;
			  m = (WHITE | KING);
			  m = m << 8;
			  m += FREE;
			  m = m << 8;
			  m += i - 10;
			  movelist[n].m[1] = m;
			  m = FREE;
			  m = m << 8;
			  m += (WHITE | KING);
			  m = m << 8;
			  m += i;
			  movelist[n].m[0] = m;
			  m = FREE;
			  m = m << 8;
			  m += b[i - 5];
			  m = m << 8;
			  m += i - 5;
			  movelist[n].m[2] = m;
			  tmp = b[i - 5];
			  b[i - 5] = FREE;
			  whitekingcapture (b, &n, movelist, i - 10);
			  b[i - 5] = tmp;
			}
		    }
		}
	    }
	}
    }
  return (n);
}

void
blackmancapture (int b[46], int *n, struct move2 movelist[MAXMOVES], int i)
{
  int m;
  int found = 0;
  struct move2 move, orgmove;

  orgmove = movelist[*n];
  move = orgmove;

  if ((b[i + 4] & WHITE) != 0)
    {
      if ((b[i + 8] & FREE) != 0)
	{
	  move.n++;
	  if (i >= 28)
	    m = (BLACK | KING);
	  else
	    m = (BLACK | MAN);
	  m = m << 8;
	  m += FREE;
	  m = m << 8;
	  m += (i + 8);
	  move.m[1] = m;
	  m = FREE;
	  m = m << 8;
	  m += b[i + 4];
	  m = m << 8;
	  m += (i + 4);
	  move.m[move.n - 1] = m;
	  found = 1;
	  movelist[*n] = move;
	  blackmancapture (b, n, movelist, i + 8);
	}
    }
  move = orgmove;
  if ((b[i + 5] & WHITE) != 0)
    {
      if ((b[i + 10] & FREE) != 0)
	{
	  move.n++;
	  if (i >= 28)
	    m = (BLACK | KING);
	  else
	    m = (BLACK | MAN);
	  m = m << 8;
	  m += FREE;
	  m = m << 8;
	  m += (i + 10);
	  move.m[1] = m;
	  m = FREE;
	  m = m << 8;
	  m += b[i + 5];
	  m = m << 8;
	  m += (i + 5);
	  move.m[move.n - 1] = m;
	  found = 1;
	  movelist[*n] = move;
	  blackmancapture (b, n, movelist, i + 10);
	}
    }
  if (!found)
    (*n)++;
}

void
blackkingcapture (int b[46], int *n, struct move2 movelist[MAXMOVES], int i)
{
  int m;
  int tmp;
  int found = 0;
  struct move2 move, orgmove;

  orgmove = movelist[*n];
  move = orgmove;

  if ((b[i - 4] & WHITE) != 0)
    {
      if ((b[i - 8] & FREE) != 0)
	{
	  move.n++;
	  m = (BLACK | KING);
	  m = m << 8;
	  m += FREE;
	  m = m << 8;
	  m += i - 8;
	  move.m[1] = m;
	  m = FREE;
	  m = m << 8;
	  m += b[i - 4];
	  m = m << 8;
	  m += i - 4;
	  move.m[move.n - 1] = m;
	  found = 1;
	  movelist[*n] = move;
	  tmp = b[i - 4];
	  b[i - 4] = FREE;
	  blackkingcapture (b, n, movelist, i - 8);
	  b[i - 4] = tmp;
	}
    }
  move = orgmove;
  if ((b[i - 5] & WHITE) != 0)
    {
      if ((b[i - 10] & FREE) != 0)
	{
	  move.n++;
	  m = (BLACK | KING);
	  m = m << 8;
	  m += FREE;
	  m = m << 8;
	  m += i - 10;
	  move.m[1] = m;
	  m = FREE;
	  m = m << 8;
	  m += b[i - 5];
	  m = m << 8;
	  m += i - 5;
	  move.m[move.n - 1] = m;
	  found = 1;
	  movelist[*n] = move;
	  tmp = b[i - 5];
	  b[i - 5] = FREE;
	  blackkingcapture (b, n, movelist, i - 10);
	  b[i - 5] = tmp;
	}
    }
  move = orgmove;
  if ((b[i + 4] & WHITE) != 0)
    {
      if ((b[i + 8] & FREE) != 0)
	{
	  move.n++;
	  m = (BLACK | KING);
	  m = m << 8;
	  m += FREE;
	  m = m << 8;
	  m += i + 8;
	  move.m[1] = m;
	  m = FREE;
	  m = m << 8;
	  m += b[i + 4];
	  m = m << 8;
	  m += i + 4;
	  move.m[move.n - 1] = m;
	  found = 1;
	  movelist[*n] = move;
	  tmp = b[i + 4];
	  b[i + 4] = FREE;
	  blackkingcapture (b, n, movelist, i + 8);
	  b[i + 4] = tmp;
	}
    }
  move = orgmove;
  if ((b[i + 5] & WHITE) != 0)
    {
      if ((b[i + 10] & FREE) != 0)
	{
	  move.n++;
	  m = (BLACK | KING);
	  m = m << 8;
	  m += FREE;
	  m = m << 8;
	  m += i + 10;
	  move.m[1] = m;
	  m = FREE;
	  m = m << 8;
	  m += b[i + 5];
	  m = m << 8;
	  m += i + 5;
	  move.m[move.n - 1] = m;
	  found = 1;
	  movelist[*n] = move;
	  tmp = b[i + 5];
	  b[i + 5] = FREE;
	  blackkingcapture (b, n, movelist, i + 10);
	  b[i + 5] = tmp;
	}
    }
  if (!found)
    (*n)++;
}

void
whitemancapture (int b[46], int *n, struct move2 movelist[MAXMOVES], int i)
{
  int m;
  int found = 0;
  struct move2 move, orgmove;

  orgmove = movelist[*n];
  move = orgmove;

  if ((b[i - 4] & BLACK) != 0)
    {
      if ((b[i - 8] & FREE) != 0)
	{
	  move.n++;
	  if (i <= 17)
	    m = (WHITE | KING);
	  else
	    m = (WHITE | MAN);
	  m = m << 8;
	  m += FREE;
	  m = m << 8;
	  m += i - 8;
	  move.m[1] = m;
	  m = FREE;
	  m = m << 8;
	  m += b[i - 4];
	  m = m << 8;
	  m += i - 4;
	  move.m[move.n - 1] = m;
	  found = 1;
	  movelist[*n] = move;
	  whitemancapture (b, n, movelist, i - 8);
	}
    }
  move = orgmove;
  if ((b[i - 5] & BLACK) != 0)
    {
      if ((b[i - 10] & FREE) != 0)
	{
	  move.n++;
	  if (i <= 17)
	    m = (WHITE | KING);
	  else
	    m = (WHITE | MAN);
	  m = m << 8;
	  m += FREE;
	  m = m << 8;
	  m += i - 10;
	  move.m[1] = m;
	  m = FREE;
	  m = m << 8;
	  m += b[i - 5];
	  m = m << 8;
	  m += i - 5;
	  move.m[move.n - 1] = m;
	  found = 1;
	  movelist[*n] = move;
	  whitemancapture (b, n, movelist, i - 10);
	}
    }
  if (!found)
    (*n)++;
}

void
whitekingcapture (int b[46], int *n, struct move2 movelist[MAXMOVES], int i)
{
  int m;
  int tmp;
  int found = 0;
  struct move2 move, orgmove;

  orgmove = movelist[*n];
  move = orgmove;

  if ((b[i - 4] & BLACK) != 0)
    {
      if ((b[i - 8] & FREE) != 0)
	{
	  move.n++;
	  m = (WHITE | KING);
	  m = m << 8;
	  m += FREE;
	  m = m << 8;
	  m += i - 8;
	  move.m[1] = m;
	  m = FREE;
	  m = m << 8;
	  m += b[i - 4];
	  m = m << 8;
	  m += i - 4;
	  move.m[move.n - 1] = m;
	  found = 1;
	  movelist[*n] = move;
	  tmp = b[i - 4];
	  b[i - 4] = FREE;
	  whitekingcapture (b, n, movelist, i - 8);
	  b[i - 4] = tmp;
	}
    }
  move = orgmove;
  if ((b[i - 5] & BLACK) != 0)
    {
      if ((b[i - 10] & FREE) != 0)
	{
	  move.n++;
	  m = (WHITE | KING);
	  m = m << 8;
	  m += FREE;
	  m = m << 8;
	  m += i - 10;
	  move.m[1] = m;
	  m = FREE;
	  m = m << 8;
	  m += b[i - 5];
	  m = m << 8;
	  m += i - 5;
	  move.m[move.n - 1] = m;
	  found = 1;
	  movelist[*n] = move;
	  tmp = b[i - 5];
	  b[i - 5] = FREE;
	  whitekingcapture (b, n, movelist, i - 10);
	  b[i - 5] = tmp;
	}
    }
  move = orgmove;
  if ((b[i + 4] & BLACK) != 0)
    {
      if ((b[i + 8] & FREE) != 0)
	{
	  move.n++;
	  m = (WHITE | KING);
	  m = m << 8;
	  m += FREE;
	  m = m << 8;
	  m += i + 8;
	  move.m[1] = m;
	  m = FREE;
	  m = m << 8;
	  m += b[i + 4];
	  m = m << 8;
	  m += i + 4;
	  move.m[move.n - 1] = m;
	  found = 1;
	  movelist[*n] = move;
	  tmp = b[i + 4];
	  b[i + 4] = FREE;
	  whitekingcapture (b, n, movelist, i + 8);
	  b[i + 4] = tmp;
	}
    }
  move = orgmove;
  if ((b[i + 5] & BLACK) != 0)
    {
      if ((b[i + 10] & FREE) != 0)
	{
	  move.n++;
	  m = (WHITE | KING);
	  m = m << 8;
	  m += FREE;
	  m = m << 8;
	  m += i + 10;
	  move.m[1] = m;
	  m = FREE;
	  m = m << 8;
	  m += b[i + 5];
	  m = m << 8;
	  m += i + 5;
	  move.m[move.n - 1] = m;
	  found = 1;
	  movelist[*n] = move;
	  tmp = b[i + 5];
	  b[i + 5] = FREE;
	  whitekingcapture (b, n, movelist, i + 10);
	  b[i + 5] = tmp;
	}
    }
  if (!found)
    (*n)++;
}
