%{
/*
 * A B O U T
 *
 * parse portable draughts notation
 * interactive mode to playback single games
 * (c) 2004 Peter Chiocchetti <pch(a)myzel.net>
 *
 * This is the parser definition.
 */

/*
 * C O P Y I N G
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "pdntool.h"

// locals
unsigned int line, col;		// line, column count in input
unsigned int ng = 1;		// number of games parsed

// prototypes
int yylex(void);

// functions
void
yyerror (PDNgame *pg, char const *msg)
{
  fprintf (stderr,
    "%s: %s at line %d before column %d.\n",
    progname, msg, line, col);
}

int
yywrap ()
{
  //line = col = ng = 1;
  return 1;
}

%}

%union {
  char *s;
  int  n;
}

%file-prefix="y"
%parse-param {PDNgame *pg}

// TERMINALS in uppercase
%token TEVENT TDATE TBLACK TWHITE TSITE TRESULT TROUND
%token TSETUP TFEN TGAMETYPE
%token <s> TUNKNOWN
%token QUOTE OBRACKET EBRACKET OBRACE EBRACE
%token <s> TURN
%token BPOS WPOS
%token <n> MPIECE KPIECE
%token <s> QSTRING CSTRING
%token CARDINAL
%token <s> JUMP MOVE RESULT
%token <s> STRENGTH

%%

line: /* empty */
	| line headers
	| line movelist
	;

headers: OBRACKET header EBRACKET
	| OBRACE CSTRING EBRACE
	{
	  pg->ma[0] = $2;
	}
	;

header:
	TEVENT QUOTE QUOTE
	| TEVENT QUOTE QSTRING QUOTE
	{
	  pg->event = $3;
	}
	| TDATE QUOTE QUOTE
	| TDATE QUOTE QSTRING QUOTE
	{
	  pg->date = $3;
	}
	| TBLACK QUOTE QUOTE
	| TBLACK QUOTE QSTRING QUOTE
	{
	  pg->black = $3;
	}
	| TWHITE QUOTE QUOTE
	| TWHITE QUOTE QSTRING QUOTE
	{
	  pg->white = $3;
	}
	| TSITE QUOTE QUOTE
	| TSITE QUOTE QSTRING QUOTE
	{
	  pg->site = $3;
	}
	| TRESULT QUOTE QUOTE
	| TRESULT QUOTE QSTRING QUOTE
	{
	  pg->result = $3;
	}
	| TROUND QUOTE QUOTE
	| TROUND QUOTE QSTRING QUOTE
	{
	  pg->round = $3;
	}
	| TFEN QUOTE QUOTE
	| TFEN QUOTE fen QUOTE
	{
	  pg->fen = 1;
	}
	| TGAMETYPE QUOTE QUOTE
	| TGAMETYPE QUOTE QSTRING QUOTE
	{
	  pg->gametype = strtol($3, NULL, 10);
	  free ($3);
	}
	| TUNKNOWN QUOTE QUOTE
	| TUNKNOWN QUOTE QSTRING QUOTE
	{
	    fprintf (stderr,
	      "%s: unknown header [%s \"%s\"] at line %d before column %d.\n",
	      progname, $1, $3, line, col);
	      free ($1);
	      free ($3);
	}
	;

fen: TURN
	{
	  pg->turn = $1;
	}
	| fen BPOS bpieces WPOS wpieces
	| fen WPOS wpieces BPOS bpieces
	;

bpieces: /* empty */
	| bpieces MPIECE
	{
	  if ($2 <= SQUARE * SQUARE / 2)
	    pg->fpos[$2] = BLACK | MAN;
	}
	| bpieces KPIECE
	{
	  if ($2 <= SQUARE * SQUARE / 2)
	    pg->fpos[$2] = BLACK | KING;
	}
	;

wpieces: /* empty */
	| wpieces MPIECE
	{
	  if ($2 <= SQUARE * SQUARE / 2)
	    pg->fpos[$2] = WHITE | MAN;
	}
	| wpieces KPIECE
	{
	  if ($2 <= SQUARE * SQUARE / 2)
	    pg->fpos[$2] = WHITE | KING;
	}
	;

movelist: moves RESULT
	{
	  if (pg->result && strcmp (pg->result, $2))
	    fprintf (stderr,
	      "%s: different results at line %d before column %d.\n",
	      progname, line, col);
	  if (!pg->result)
	    pg->result = $2;
	  else
	    free ($2);
	  pg->ng = ng;
	  gotgame (pg);
	  ++ng;
	}
	| moves error
	{
	  if (!pg->result)
	    fprintf (stderr,
	      "%s: no result at line %d before column %d.\n",
	      progname, line, col);
	  fprintf(stderr,
	    "%s: movelist doesnt end with a result.\n",
	     progname);
	  pg->ng = ng;
	  gotgame (pg);
	  ++ng;
	}
	;

moves: movenum move strength annotation
	{
	  ++pg->nm;
	}
	| moves movenum move strength annotation
	{
	  ++pg->nm;
	}
	;

movenum: /* empty */
	| CARDINAL
	;

move: MOVE
	{
	  pg->mv[pg->nm] = $1;
	}
	| JUMP
	{
	  pg->mv[pg->nm] = $1;
	}
	;

strength: /* empty */
	| STRENGTH
	{
	   pg->ms[pg->nm] = $1;
	}
	;

annotation: /* empty */
	| OBRACE CSTRING EBRACE
	{
	  pg->ma[pg->nm] = $2;
	}
	;
